import sqlite3
import time
import grpc
import subprocess  # Import subprocess to execute shell commands
from chirpstack_api import api

# Configuration.
server = "localhost:8080"
# Get the API token from the shell command using the correct key name
api_token = subprocess.check_output(["uci", "get", "modbus.chirpstack.api_token"]).strip().decode('utf-8')

def check_for_new_data(last_row_id):
    conn = sqlite3.connect('/usr/local/rs485/downlink.db')
    cursor = conn.cursor()

    cursor.execute("SELECT id, register_start, downlink_data FROM modbus_downlink WHERE id > ?", (last_row_id,))
    row = cursor.fetchone()
    new_data = None  # initialize new_data

    if row:
        current_row_id, register_start, downlink_data = row
        new_data = {'register_start': register_start, 'downlink_data': downlink_data}
        last_row_id = current_row_id

    conn.close()
    return last_row_id, new_data  # return last_row_id and new_data

def get_dev_eui(register_start):
    conn = sqlite3.connect('/usr/local/rs485/downlink.db')
    cursor = conn.cursor()

    cursor.execute("SELECT dev_eui FROM downlink_setting WHERE register_start = ?", (register_start,))
    result = cursor.fetchone()
    
    conn.close()

    if result:
        return result[0]  # Return the dev_eui
    return None

def delete_sent_data(row_id):
    conn = sqlite3.connect('/usr/local/rs485/downlink.db')
    cursor = conn.cursor()

    cursor.execute("DELETE FROM modbus_downlink WHERE id = ?", (row_id,))
    conn.commit()  # commit the deletion
    #print(f"Removed ID for {row_id} ")

    conn.close()

def enqueue_downlink(dev_eui, data, row_id):
    # Connect without using TLS.
    channel = grpc.insecure_channel(server)

    # Device-queue API client.
    client = api.DeviceServiceStub(channel)

    # Define the API key meta-data.
    auth_token = [("authorization", "Bearer %s" % api_token)]

    # Construct request.
    req = api.EnqueueDeviceQueueItemRequest()
    req.queue_item.confirmed = False
    req.queue_item.dev_eui = dev_eui.hex()  # Convert dev_eui to hex
    req.queue_item.f_port = 2
    req.queue_item.data = data

    try:
        resp = client.Enqueue(req, metadata=auth_token)
        print(f"Downlink ID: {resp.id}")
    except grpc.RpcError as e:
        print(f"gRPC Error: {e.code()}: {e.details()}")

    delete_sent_data(row_id)

def main():
    last_row_id = 0
    while True:
        last_row_id, new_data = check_for_new_data(last_row_id)
        if new_data:
            # Get register_start from the new data
            register_start = new_data['register_start']
            
            # Find matching dev_eui based on register_start
            dev_eui = get_dev_eui(register_start)
            if dev_eui:
                # Check data length and send downlink if valid
                downlink_data = new_data['downlink_data']
                byte_data = bytes.fromhex(downlink_data.replace('0x', '').replace(' ', ''))
                if len(byte_data) >= 2:  # Ensure there is enough data to send
                    item_data = byte_data  # Full data to be sent as downlink

                    # Convert item_data to byte data in 0x format for logging
                    item_data_hex = ', '.join([f"0x{byte:02X}" for byte in item_data])
                    print(f"downlink_hex: {item_data_hex}")

                    # Pass item_data and the current row's row_id directly
                    enqueue_downlink(bytes.fromhex(dev_eui), item_data, last_row_id)
                else:
                    print("Incorrect data format, can't send downlink.")
            else:
                print(f"No matching dev_eui found for register_start: {register_start}")
        time.sleep(1)

if __name__ == "__main__":
    main()
