const mqtt = require('mqtt');
const sqlite3 = require('sqlite3').verbose();

const brokerUrl = 'mqtt://127.0.0.1';
const options = {
    clientId: 'mqtt-subscriber',
    clean: true,
};

const client = mqtt.connect(brokerUrl, options);

client.on('connect', () => {
    console.log('Connected to MQTT broker');
    const topic = 'application/+/device/+/event/up';
    client.subscribe(topic, (err) => {
        if (!err) {
            console.log(`Subscribed to topic: ${topic}`);
        } else {
            console.error('Subscription error:', err);
        }
    });
});

client.on('message', (topic, message) => {
	const parsedData = JSON.parse(message);
	const date = Math.floor(new Date(parsedData.time).getTime() / 1000);
	const date_h = date.toString(16);
	const deui =  parsedData.deviceInfo.devEui
	const dname = parsedData.deviceInfo.deviceName
	const addr = parsedData.devAddr
	const hexadecimalData = Buffer.from(parsedData.data, 'base64').toString('hex');
	const fcnt = parsedData.fCnt
    const fport = parsedData.fPort
    const rssi = parsedData.rxInfo[0].rssi
    const rssi_hex = "00" + Math.abs(rssi).toString(16);
	const decode_data = parsedData.object
	const db = new sqlite3.Database('/usr/local/rs485/data.db', (err) => {
        if (err) {
            return console.error(err.message);
        }
    //console.log('Connected to the SQLite database');

    const tableToCheck = `table${deui}`;
    //console.log(tableToCheck);
    const query2 = "SELECT fport FROM listening  WHERE dev_eui = ?;";
    db.get(query2, [deui], (err, row) => {
        if (err) {
            console.error(err.message);
            db.close();
            return;
        }
        if (row) {
            db.run(`
                CREATE TABLE IF NOT EXISTS ${tableToCheck} (
                    id INTEGER PRIMARY KEY,
                    dev_eui TEXT,
                    dev_addr TEXT,
                    fcnt INTEGER,
                    rssi TEXT,
                    payload TEXT,
                    date TEXT
                )
            `, (err) => {
                if (err) {
                    console.error(err.message);
                    db.close();
                    return;
                }
                db.run(
                    `INSERT INTO ${tableToCheck} (dev_eui, dev_addr, fcnt, rssi, payload, date) VALUES (?, ?, ?, ?, ?, ?)`,
                    [deui, addr, fcnt, rssi_hex, hexadecimalData, date_h],
                    function (err) {
                        if (err) {
                            console.error(err.message);
                        }
                        db.close();
                    }
                );
                db.serialize(function() {
                    const stmt = db.prepare("UPDATE maindata SET dev_addr = ?, fcnt = ?, payload = ?, date = ?, rssi = ? WHERE dev_eui = ?");
                    stmt.run(addr,fcnt, hexadecimalData, date_h, rssi_hex, deui); // update 'a84041'
                    stmt.finalize();
                });
            });

            db.run('CREATE TABLE IF NOT EXISTS decode_data (dev_eui TEXT PRIMARY KEY, data TEXT, fport TEXT)', (err) => {
                if (err) {
                    console.error(err.message);
                    db.close();
                    return;
                }
                db.get("SELECT * FROM decode_data WHERE dev_eui = ?", [deui], (err, row) => {
                    if (err) {
                        console.error("检查现有条目时出错:", err);
                        return;
                    }
                    if (!row) {
                        db.run("INSERT INTO decode_data (dev_eui, data) VALUES (?, ?)", [deui, JSON.stringify(decode_data)], (err) => {
                            if (err) {
                                console.error('初始化数据库时出错:', err);
                            }
                        });
                    }
                });
            });

            const jsonString = JSON.stringify(decode_data);
            if (!jsonString) {
                return console.error('Invalid JSON string');
            }
                const fport_query = row.fport;
                if (fport_query != "" && fport != fport_query) {
                    return;
                }
                db.run("UPDATE decode_data SET data = ? WHERE dev_eui = ?", [jsonString, deui], (err) => {
                    if (err) {
                        console.error('Error updating database:', err);
                    }
                });
        } else {
            db.close();
        }
        });
    });
});
client.on('close', () => {
    console.log('Connection closed');
});
